@@ -970,6 +970,10 @@ load_rownames_dataset_attribute (SEXP call, hid_t dataset, SEXP vec)
       }      
       setAttrib (vec, R_RowNamesSymbol, rownames);
       UNPROTECT (1);
+
+      if (H5Sclose (space) < 0)
+	errorcall (call, "unable to close dataset space");
+
       return;
     }
 
@@ -1003,9 +1007,21 @@ load_rownames_dataset_attribute (SEXP call, hid_t dataset, SEXP vec)
 
     for (ri = 0; ri < rowcount; ri++)
       SET_STRING_ELT (rownames, ri, strptrs [ri]);
+
+    if (H5Tclose (rtid) < 0)
+      errorcall (call, "can't close close reference type");
   }
   setAttrib (vec, R_RowNamesSymbol, rownames);
   UNPROTECT (1);
+
+  if (H5Sclose (rnspace) < 0)
+    errorcall (call, "unable to close row name space");
+  
+  if (H5Tclose (rntid) < 0)
+    errorcall (call, "unable to close row name type");
+    
+  if (H5Aclose (rnattrib) < 0)
+    errorcall (call, "unable to close row name attribute");
 }
 
 struct hdf5_attribute_info {
@@ -1186,7 +1202,6 @@ hdf5_process_object (hid_t id, const char *name, void *client_data)
       hid_t dataset, space, tid;
       int rank;
       SEXPTYPE type = NILSXP;
-      /*H5T_class_t class;*/
 
       if (hdf5_global_verbosity > 0)
         Rprintf ("... its a dataset...");
@@ -1294,7 +1309,7 @@ hdf5_process_object (hid_t id, const char *name, void *client_data)
               
               if (hdf5_global_verbosity > 2)
                 Rprintf ("....done\n");
-              
+
               PROTECT (vec = allocVector (VECSXP, colcount));
               PROTECT (names = allocVector (STRSXP, colcount));
               
@@ -1324,6 +1339,7 @@ hdf5_process_object (hid_t id, const char *name, void *client_data)
                     
                     if (colname)
                       SET_STRING_ELT (names, ci, mkChar (colname));
+                    free (colname);
                   }
                   switch (class)
                     {
@@ -1347,6 +1363,8 @@ hdf5_process_object (hid_t id, const char *name, void *client_data)
                       errorcall (iinfo->call, "can't handle hdf class %d",
                                  class);
                     }
+                  if (H5Tclose (ctid) < 0)
+                    errorcall (iinfo->call, "could not close member type");
                 }
               load_rownames_dataset_attribute (iinfo->call, dataset, vec);
               setAttrib (vec, R_NamesSymbol, names);
@@ -1355,6 +1373,7 @@ hdf5_process_object (hid_t id, const char *name, void *client_data)
               iinfo->add (iinfo, name, vec);
               hdf5_load_attributes (iinfo->call, dataset, vec, name);
               UNPROTECT (1);
+
               if (H5Tclose (rtid) < 0)
                 errorcall (iinfo->call, "could not close reference type");
             }
